# -*- coding: utf-8 -*-
"""Météo-France helpers."""
import math
from datetime import datetime
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple

from pytz import timezone
from pytz import utc

from .const import ALERT_COLOR_LIST_EN
from .const import ALERT_COLOR_LIST_FR
from .const import ALERT_TYPE_LIST_EN
from .const import ALERT_TYPE_LIST_FR
from .const import COASTAL_DEPARTMENT_LIST
from .const import VALID_DEPARTMENT_LIST
from .model.place import Place


def get_warning_text_status_from_indice_color(
    int_color: int, lang: str = "fr"
) -> Optional[str]:
    """Convert the color code (in int) in readable text (Helper).

    Returned text is in French or English according to the lang parameter.
    """
    if lang == "fr":
        return ALERT_COLOR_LIST_FR[int_color]

    return ALERT_COLOR_LIST_EN[int_color]


def get_phenomenon_name_from_indice(
    int_phenomenon: int, lang: str = "fr"
) -> Optional[str]:
    """Convert the phenomenom code in readable text (Hepler).

    Returned text is in French or English according to the lang parameter.
    """
    if lang == "fr":
        return ALERT_TYPE_LIST_FR[int_phenomenon]

    return ALERT_TYPE_LIST_EN[int_phenomenon]


def is_coastal_department(department_number: str) -> bool:
    """Identify when a second bulletin is availabe for coastal risks (Helper)."""
    return department_number in COASTAL_DEPARTMENT_LIST


def is_valid_warning_department(department_number: str) -> bool:
    """Identify if there is a weather alert bulletin for this department (Helper).

    Weather alert buletins are available only for France métropole and Andorre.
    """
    return department_number in VALID_DEPARTMENT_LIST


def readeable_phenomenoms_dict(
    list_phenomenoms: List[Dict[str, int]], language: str = "fr"
) -> Dict[Optional[str], Optional[str]]:
    """Create a dictionary with human readable keys and values (Helper)."""
    # Init empty dictionnary
    readable_dict = {}

    # Translate phenomenom name and alert level
    for phenomenom in list_phenomenoms:
        readable_dict[
            get_phenomenon_name_from_indice(phenomenom["phenomenon_id"], language)
        ] = get_warning_text_status_from_indice_color(
            phenomenom["phenomenon_max_color_id"], language
        )
    return readable_dict


def haversine(coord1: Tuple[float, float], coord2: Tuple[float, float]) -> float:
    """Compute distance in meters between to GPS coordinates using Harvesine formula.

    coord1 and coord2 are tuple with latitude and longitude in degrees.
    source: https://janakiev.com/blog/gps-points-distance-python/
    """
    radius = 6372800  # Earth radius in meters
    lat1, lon1 = coord1
    lat2, lon2 = coord2
    phi1, phi2 = math.radians(lat1), math.radians(lat2)
    dphi = math.radians(lat2 - lat1)
    dlambda = math.radians(lon2 - lon1)

    a = (
        math.sin(dphi / 2) ** 2
        + math.cos(phi1) * math.cos(phi2) * math.sin(dlambda / 2) ** 2
    )

    return 2 * radius * math.atan2(math.sqrt(a), math.sqrt(1 - a))


def sort_places_versus_distance_from_coordinates(
    list_places: List[Place], gps_coord: Tuple[float, float]
) -> List[Place]:
    """Oder list of places according to the distance to a reference coordinates.

    gps_coordinates are in degrees.
    Note: this helper is compensating the bad results of the API. Results in the API
    are sorted but lot of case identified where it doesn't work (example: Montréal)
    """
    sorted_places = sorted(
        list_places,
        key=lambda x: haversine((float(x.latitude), float(x.longitude)), gps_coord),
    )
    return sorted_places


def timestamp_to_dateime_with_locale_tz(timestamp: int, local_tz: str) -> datetime:
    """Convert timestamp in datetime (Helper).

    The local timezone corresponding to the forecast location is used.
    """
    # convert timestamp in datetime with UTC timezone
    dt_utc = utc.localize(datetime.utcfromtimestamp(timestamp))
    # convert datetime to local timezone
    return dt_utc.astimezone(timezone(local_tz))



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API. Classes for weather alert.

For getting weather alerts in France and Andorre.
"""
import sys
from typing import Any
from typing import Dict
from typing import List

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class WarnningCurrentPhenomenonsData(TypedDict):
    """Describing the structure of the API returned CurrentPhenomenons object."""

    update_time: int
    end_validity_time: int
    domain_id: str
    phenomenons_max_colors: List[Dict[str, int]]


class WarnningFullData(TypedDict):
    """Describing the structure of the API returned full object."""

    update_time: int
    end_validity_time: int
    domain_id: str
    color_max: int
    timelaps: List[Dict[str, Any]]
    phenomenons_items: List[Dict[str, int]]
    advices: List[Dict[str, Any]]
    consequences: List[Dict[str, Any]]
    max_count_items: Any  # Didn't see any value yet
    comments: Dict[str, Any]
    text: Dict[str, Any]
    text_avalanche: Any  # Didn't see any value yet


class CurrentPhenomenons:
    """Class to access the results of a `warning/currentPhenomenons` API command.

    For coastal department two bulletins are avalaible corresponding to two different
    domains.
    """

    def __init__(self, raw_data: WarnningCurrentPhenomenonsData):
        """Initialize a CurrentPhenomenons object."""
        self.raw_data = raw_data

    @property
    def update_time(self) -> int:
        """Return the update time of the phenomenoms."""
        return self.raw_data["update_time"]

    @property
    def end_validity_time(self) -> int:
        """Return the end of validty time of the phenomenoms."""
        return self.raw_data["end_validity_time"]

    @property
    def domain_id(self) -> str:
        """Return the domain ID of the phenomenoms."""
        return self.raw_data["domain_id"]

    @property
    def phenomenons_max_colors(self) -> List[Dict[str, int]]:
        """Return the list and colors of the phenomenoms."""
        return self.raw_data["phenomenons_max_colors"]

    def merge_with_coastal_phenomenons(
        self, coastal_phenomenoms: "CurrentPhenomenons"
    ) -> None:
        """Merge the classical phenomenoms bulleting with the coastal one."""
        # TODO: Add consitency check
        self.raw_data["phenomenons_max_colors"].extend(
            coastal_phenomenoms.phenomenons_max_colors
        )

    def get_domain_max_color(self) -> int:
        """Get the maximum level of alert of a given domain (class helper)."""
        max_int_color = max(
            x["phenomenon_max_color_id"] for x in self.phenomenons_max_colors
        )
        return max_int_color


class Full:
    """This class allows to access the results of a `warning/full` API command.

    For a given domain we can access the maximum alert, a timelaps of the alert
    evolution for the next 24 hours, and a list of alerts.

    For coastal department two bulletins are avalaible corresponding to two different
    domains.
    """

    def __init__(self, raw_data: WarnningFullData):
        """Initialize a Full object."""
        self.raw_data = raw_data

    @property
    def update_time(self) -> int:
        """Return the update time of the full bulletin."""
        return self.raw_data["update_time"]

    @property
    def end_validity_time(self) -> int:
        """Return the end of validty time of the full bulletin."""
        return self.raw_data["end_validity_time"]

    @property
    def domain_id(self) -> str:
        """Return the domain ID of the the full bulletin."""
        return self.raw_data["domain_id"]

    @property
    def color_max(self) -> int:
        """Return the color max of the domain."""
        return self.raw_data["color_max"]

    @property
    def timelaps(self) -> List[Dict[str, Any]]:
        """Return the timelaps of each phenomenom for the domain."""
        return self.raw_data["timelaps"]

    @property
    def phenomenons_items(self) -> List[Dict[str, int]]:
        """Return the phenomenom list of the domain."""
        return self.raw_data["phenomenons_items"]

    def merge_with_coastal_phenomenons(self, coastal_phenomenoms: "Full") -> None:
        """Merge the classical phenomenoms bulleting with the coastal one."""
        # TODO: Add consitency check
        # TODO: Check if other data need to be merged

        # Merge color_max property
        self.raw_data["color_max"] = max(self.color_max, coastal_phenomenoms.color_max)

        # Merge timelaps
        self.raw_data["timelaps"].extend(coastal_phenomenoms.timelaps)

        # Merge phenomenons_items
        self.raw_data["phenomenons_items"].extend(coastal_phenomenoms.phenomenons_items)

    # TODO: check opportunity to complete class



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API. PictureOfTheDay class."""
import sys
from typing import Mapping

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class PictureOfTheDayData(TypedDict):
    """Describe the structure of the API returned ImageJour object."""

    vignette: str
    imageHD: str  # noqa: N815
    commentaire: str


class PictureOfTheDay:
    """Class to access the results of a `ImageJour/last` API command."""

    def __init__(self, raw_data: Mapping[str, PictureOfTheDayData]):
        """Initialize a PictureOfTheDay object."""
        self.raw_data = raw_data["result"]

    @property
    def image_url(self) -> str:
        """Return the image URL of the picture of the day."""
        return self.raw_data["vignette"]

    @property
    def image_hd_url(self) -> str:
        """Return the image HD URL of the picture of the day."""
        return self.raw_data["imageHD"]

    @property
    def description(self) -> str:
        """Return the description of the picture of the day."""
        return self.raw_data["commentaire"]



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API. Rain class."""
import sys
from datetime import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict

from meteofrance.helpers import timestamp_to_dateime_with_locale_tz


class RainData(TypedDict):
    """Describing the structure of the API returned rain object."""

    position: Dict[str, Any]
    updated_on: int
    forecast: List[Dict[str, Any]]
    quality: int


class Rain:
    """Class to access the results of 'rain' API command."""

    def __init__(self, raw_data: RainData):
        """Initialize a Rain object."""
        self.raw_data = raw_data

    @property
    def position(self) -> Dict[str, Any]:
        """Return the position information of the rain forecast."""
        return self.raw_data["position"]

    @property
    def updated_on(self) -> int:
        """Return the update timestamp of the rain forecast."""
        return self.raw_data["updated_on"]

    @property
    def forecast(self) -> List[Dict[str, Any]]:
        """Return the rain forecast."""
        return self.raw_data["forecast"]

    @property
    def quality(self) -> int:
        """Return the quality of the rain forecast."""
        # TODO: don't know yet what is the usage
        return self.raw_data["quality"]

    def next_rain_date_locale(self) -> Optional[datetime]:
        """Return the date of the next rain in the Place timezone (Helper)."""
        # search first cadran with rain
        next_rain = next(
            (cadran for cadran in self.forecast if cadran["rain"] > 1), None
        )

        next_rain_dt_local: Optional[datetime] = None
        if next_rain is not None:
            # get the time stamp of the first cadran with rain
            next_rain_timestamp = next_rain["dt"]
            # convert timestamp in datetime with local timezone
            next_rain_dt_local = timestamp_to_dateime_with_locale_tz(
                next_rain_timestamp, self.position["timezone"]
            )

        return next_rain_dt_local

    def timestamp_to_locale_time(self, timestamp: int) -> datetime:
        """Convert timestamp in datetime (Helper).

        The timezone corresponding to the forecast location is used.
        """
        return timestamp_to_dateime_with_locale_tz(timestamp, self.position["timezone"])



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API. Forecast class."""
import sys
from datetime import datetime
from typing import Any
from typing import Dict
from typing import List

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict

from pytz import utc

from meteofrance.helpers import timestamp_to_dateime_with_locale_tz


class ForecastData(TypedDict):
    """Describing the structure of the API returned forecast object."""

    position: Dict[str, Any]
    updated_on: int
    daily_forecast: List[Dict[str, Any]]
    forecast: List[Dict[str, Any]]
    probability_forecast: List[Dict[str, Any]]


class Forecast:
    """Class to access the results of a `forecast` API command."""

    def __init__(self, raw_data: ForecastData):
        """Initialize a Forecast object."""
        self.raw_data = raw_data

    @property
    def position(self) -> Dict[str, Any]:
        """Return the position information of the forecast."""
        return self.raw_data["position"]

    @property
    def updated_on(self) -> int:
        """Return the update timestamp of the forecast."""
        return self.raw_data["updated_on"]

    @property
    def daily_forecast(self) -> List[Dict[str, Any]]:
        """Return the daily forecast for the following days."""
        return self.raw_data["daily_forecast"]

    @property
    def forecast(self) -> List[Dict[str, Any]]:
        """Return the hourly forecast."""
        return self.raw_data["forecast"]

    @property
    def probability_forecast(self) -> List[Dict[str, Any]]:
        """Return the wheather event forecast."""
        return self.raw_data.get("probability_forecast", [])

    @property
    def today_forecast(self) -> Dict[str, Any]:
        """Return the forecast for today."""
        return self.daily_forecast[0]

    @property
    def nearest_forecast(self) -> Dict[str, Any]:
        """Return the nearest hourly forecast."""
        # get timestamp for current time
        now_timestamp = int(utc.localize(datetime.utcnow()).timestamp())
        # sort list of foerecast by distance between current timestamp and
        # forecast timestamp
        sorted_forecast = sorted(
            self.forecast, key=lambda x: abs(x["dt"] - now_timestamp)
        )
        return sorted_forecast[0]

    @property
    def current_forecast(self) -> Dict[str, Any]:
        """Return the forecast of the current hour."""
        # Get the timestamp for the current hour.
        current_hour_timestamp = int(
            utc.localize(
                datetime.utcnow().replace(minute=0, second=0, microsecond=0)
            ).timestamp()
        )
        # create a dict using timestamp as keys
        forecast_by_datetime = {item["dt"]: item for item in self.forecast}
        # Return the forecast corresponding to the timestamp of the current hour if
        # exists. If not exists, returns the nearest forecast (not France countries)
        return forecast_by_datetime.get(current_hour_timestamp, self.nearest_forecast)

    def timestamp_to_locale_time(self, timestamp: int) -> datetime:
        """Convert timestamp in datetime (Helper).

        The timezone corresponding to the forecast location is used.
        """
        return timestamp_to_dateime_with_locale_tz(timestamp, self.position["timezone"])



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API. Place class.

Example of a place in JSON format:
{
   "insee":"11254",
   "name":"Montréal",
   "lat":43.2,
   "lon":2.14083,
   "country":"FR",
   "admin":"Languedoc-Roussillon",
   "admin2":"11",
   "postCode":"11290"
}
"""
import sys
from typing import Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict  # pylint: disable=no-name-in-module
else:
    from typing_extensions import TypedDict


class PlaceData(TypedDict):
    """Describe the structure of the API returned place object."""

    insee: str
    name: str
    lat: float
    lon: float
    country: str
    admin: str
    admin2: str
    postCode: str  # noqa: N815


class Place:
    """Class to access the results of 'places' API command."""

    def __init__(self, raw_data: PlaceData):
        """Initialize a Place object."""
        self.raw_data = raw_data

    def __repr__(self) -> str:
        """Return string representation of this class."""
        return "<{}(name={}, country={}, admin={})>".format(
            self.__class__.__name__, self.name, self.country, self.admin
        )

    def __str__(self) -> str:
        """Provide an easy way to identify the Place.

        examples: `Toulouse - (31)` or `Montréal - (Quebec)`
        """
        if self.country == "FR":
            return f"{self.name} - {self.admin} ({self.admin2}) - {self.country}"

        return f"{self.name} - {self.admin} - {self.country}"

    @property
    def insee(self) -> Optional[str]:
        """Return the INSEE ID of the place."""
        return self.raw_data.get("insee")

    @property
    def name(self) -> str:
        """Return the name of the place."""
        return self.raw_data["name"]

    @property
    def latitude(self) -> float:
        """Return the latitude of the place."""
        return self.raw_data["lat"]

    @property
    def longitude(self) -> float:
        """Return the longitude of the place."""
        return self.raw_data["lon"]

    @property
    def country(self) -> str:
        """Return the country code of the place."""
        return self.raw_data["country"]

    @property
    def admin(self) -> Optional[str]:
        """Return the admin of the place.

        Seems to be the department in text ex: "Gers".
        """
        return self.raw_data.get("admin")

    @property
    def admin2(self) -> Optional[str]:
        """Return the admin2 of the place.

        Seems to be the department in numbers "32".
        """
        return self.raw_data.get("admin2")

    @property
    def postal_code(self) -> Optional[str]:
        """Return the postal code of the place."""
        return self.raw_data.get("postCode")



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API."""
from typing import List
from typing import Optional

from .const import COASTAL_DEPARTMENT_LIST
from .const import METEOFRANCE_API_TOKEN
from .const import METEOFRANCE_API_URL
from .model import CurrentPhenomenons
from .model import Forecast
from .model import Full
from .model import PictureOfTheDay
from .model import Place
from .model import Rain
from .session import MeteoFranceSession
from .session import MeteoFranceWSSession
from .session import MeteoNetSession

# TODO: http://webservice.meteofrance.com/observation
# TODO: investigate bulletincote, montagne, etc...
#       http://ws.meteofrance.com/ws//getDetail/france/330630.json
# TODO: add protection for warning if domain not valid
# TODO: strategy for HTTP errors
# TODO: next rain in minute. Necessary ?
# TODO: forecast/metadata from ID to get gps ?


class MeteoFranceClient:
    """Proxy to the Météo-France API.

    You will find methods and helpers to request weather forecast, rain forecast and
    weather alert bulletin.
    """

    def __init__(self, access_token: Optional[str] = None):
        """Initialize the API and store the auth so we can make requests."""
        self.session = MeteoFranceSession(access_token)
        self.session_ws = MeteoFranceWSSession()
        self.session_net = MeteoNetSession()

    #
    # Place
    #
    def search_places(
        self,
        search_query: str,
        latitude: Optional[str] = None,
        longitude: Optional[str] = None,
    ) -> List[Place]:
        """Return the places link to a search.

        You can add GPS coordinates in parameter to search places arround a given
        location.
        """
        # Construct the list of the GET parameters
        params = {"q": search_query}
        if latitude is not None:
            params["lat"] = latitude
        if longitude is not None:
            params["lon"] = longitude

        # Send the API resuest
        resp = self.session.request("get", "places", params=params)
        return [Place(place_data) for place_data in resp.json()]

    #
    # Forecast
    #
    def get_forecast(
        self, latitude: float, longitude: float, language: str = "fr",
    ) -> Forecast:
        """Return the weather forecast for a GPS location.

        Results can be fetched in french or english according to the language parameter.
        """
        # TODO: add possibility to request forecast from id

        # Send the API request
        resp = self.session.request(
            "get",
            "forecast",
            params={"lat": latitude, "lon": longitude, "lang": language},
        )
        return Forecast(resp.json())

    def get_forecast_for_place(self, place: Place, language: str = "fr",) -> Forecast:
        """Return the weather forecast for a Place.

        Results can be fetched in french or english according to the language parameter.
        """
        return self.get_forecast(place.latitude, place.longitude, language)

    #
    # Rain
    #
    def get_rain(self, latitude: float, longitude: float, language: str = "fr") -> Rain:
        """Return the next 1 hour rain forecast for a GPS the location.

        Results can be fetched in french or english according to the language parameter.
        """
        # TODO: add protection if no rain forecast for this position

        # Send the API request
        resp = self.session.request(
            "get", "rain", params={"lat": latitude, "lon": longitude, "lang": language}
        )
        return Rain(resp.json())

    #
    # Warning
    #
    def get_warning_current_phenomenoms(
        self, domain: str, depth: int = 0, with_costal_bulletin: bool = False
    ) -> CurrentPhenomenons:
        """Return the current weather phenomenoms (or alerts) for a given domain.

        domain: could be `france` or any department numbers on two digits.
        For some departments you ca access an additional bulletin for coastal
        phenomenoms.
        To access it add `10` after the domain id (example: `1310`).

        with_costal_bulletin: If set to True, you can get the basic bulletin and
        coastal bulletin merged.

        depth: use 1 with `france` domain to have all sub location phenomenoms.
        """
        # Send the API request
        resp = self.session.request(
            "get",
            "warning/currentphenomenons",
            params={"domain": domain, "depth": depth},
        )

        # Create object with API response
        phenomenoms = CurrentPhenomenons(resp.json())

        # if user ask to have the coastal bulletin merged
        if with_costal_bulletin:
            if domain in COASTAL_DEPARTMENT_LIST:
                resp = self.session.request(
                    "get",
                    "warning/currentphenomenons",
                    params={"domain": domain + "10"},
                )
                phenomenoms.merge_with_coastal_phenomenons(
                    CurrentPhenomenons(resp.json())
                )

        return phenomenoms

    def get_warning_full(self, domain: str, with_costal_bulletin: bool = False) -> Full:
        """Return a complete bulletin of the weather phenomenons for a given domain.

        For a given domain we can access the maximum alert, a timelaps of the alert
        evolution for
        the next 24 hours, a list of alerts and other metadatas.

        domain: could be `france` or any department numbers on two digits.
        For some department you can access an additional bulletin for coastal
        phenomenoms.
        To access it add `10` after the domain id (example: `1310`).

        with_costal_bulletin: If set to True, you can get the basic bulletin and
        coastal bulletin merged.
        """
        # TODO: add formatDate parameter

        # Send the API request
        resp = self.session.request("get", "warning/full", params={"domain": domain})

        # Create object with API response
        full_phenomenoms = Full(resp.json())

        # if user ask to have the coastal bulletin merged
        if with_costal_bulletin:
            if domain in COASTAL_DEPARTMENT_LIST:
                resp = self.session.request(
                    "get", "warning/full", params={"domain": domain + "10"},
                )
                full_phenomenoms.merge_with_coastal_phenomenons(Full(resp.json()))

        return full_phenomenoms

    def get_warning_thumbnail(self, domain: str = "france") -> str:
        """Return the thumbnail of the weather phenomenoms or alerts map."""
        # Return directly the URL of the gif image
        return (
            f"{METEOFRANCE_API_URL}/warning/thumbnail?&token={METEOFRANCE_API_TOKEN}"
            f"&domain={domain}"
        )

    #
    # Picture of the day
    #
    def get_picture_of_the_day(self) -> PictureOfTheDay:
        """Return the picture of the day image URL & description."""
        resp = self.session_net.request("get", "ImageJour/last.txt")
        return PictureOfTheDay(resp.json())



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['meteofrance', 'meteofrance.model']

package_data = \
{'': ['*']}

install_requires = \
['pytz>=2020.1,<2021.0', 'requests>=2.24.0,<3.0.0']

extras_require = \
{':python_version >= "3.7" and python_version < "3.9"': ['typing-extensions>=3.7.4,<4.0.0']}

setup_kwargs = {
    'name': 'meteofrance-api',
    'version': '0.1.1',
    'description': 'Python client for Météo-France API.',
    'long_description': "# meteofrance-api\n\nClient Python pour l'API Météo-France. | Python client for Météo-France API.\n\n[![Build Status][build-shield]][build]\n[![codecov][codecov-shield]][codecov]\n[![License][license-shield]](LICENSE)\n\n[![GitHub Release][releases-shield]][releases]\n[![PyPI version][pypi-shield]][pypi]\n[![GitHub Activity][commits-shield]][commits]\n\nYou will find English README content [here](#for-english-speaking-users).\n\nVous trouverez le contenu francophone du README [ici](#pour-les-francophones).\n\n## Pour les francophones\n\n### Description\n\nCe package Python permet de gérer la communication avec l'API non publique de\nMétéo-France utilisée par les applications moblies officielles.\n\nLe client permet:\n\n- Rechercher des lieux de prévisions.\n- Accéder aux prévisions météorologiques horraires ou quotidiennes.\n- Accéder aux prévisions de pluie dans l'heure quand disponibles.\n- Accéder aux alertes météo pour chaque département français et l'Andorre. Deux\n  bulletins sont disponibles : un synthétique et un second avec l'évolution des alertes\n  pour les prochaines 24 heures (exemple [ici](http://vigilance.meteofrance.com/Bulletin_sans.html?a=dept32&b=2&c=)).\n\nCe package a été développé avec l'intention d'être utilisé par [Home-Assistant](https://home-assistant.io/) mais il peut être utilsé dans d'autres contextes.\n\n### Installation\n\nPour utiliser le module Python `meteofrance` vous devez en premier installer\nle package:\n\n`pip install meteofrance-api`\n\nVous pouvez trouver un exemple d'usage dans un module Python en regardant [le test d'intégration](tests/test_integrations.py).\n\n### Contribuer\n\nLes contributions sont les bienvenues. Veuillez consulter les bonnes pratiques\ndétaillées dans [`CONTRIBUTING.md`](CONTRIBUTING.md).\n\n## For English speaking users\n\n### Descritption\n\nThis Python package manages the communication with the private Météo-France API\nused by the official moblie applications.\n\nThe client allows:\n\n- Search a forecast location.\n- Fetch daily or hourly weather forecast.\n- Fetch rain forecast within the next hour if available.\n- Fetch the weather alerts or phenomenoms for each French department or Andorre.\n  Two bulletin are availabe: one basic and an other advanced with the timelaps evolution for the next 24 hours (example [here](http://vigilance.meteofrance.com/Bulletin_sans.html?a=dept32&b=2&c=)).\n\nThis package have been developed to be used with [Home-Assistant](https://home-assistant.io/) but it can be used in other contexts.\n\n### Installation\n\nTo use the `meteofrance` Python module, you have to install this package first:\n\n`pip install meteofrance-api`\n\nYou will find an example ot usage in a Python program in the [integration test](tests/test_integrations.py).\n\n### Contributing\n\nContributions are welcomed. Please check the guidelines in [`CONTRIBUTING.md`](CONTRIBUTING.md).\n\n[commits-shield]: https://img.shields.io/github/commit-activity/y/hacf-fr/meteofrance-api.svg?style=for-the-badge\n[commits]: https://github.com/hacf-fr/meteofrance-api/commits/master\n[license-shield]: https://img.shields.io/github/license/hacf-fr/meteofrance-api.svg?style=for-the-badge\n[releases-shield]: https://img.shields.io/github/release/hacf-fr/meteofrance-api.svg?style=for-the-badge\n[releases]: https://github.com/hacf-fr/meteofrance-api/releases\n[build-shield]: https://img.shields.io/github/workflow/status/hacf-fr/meteofrance-api/Python%20package?style=for-the-badge\n[build]: https://github.com/hacf-fr/meteofrance-api/actions?query=workflow%3A%22Python+package%22\n[codecov-shield]: https://img.shields.io/codecov/c/github/hacf-fr/meteofrance-api?style=for-the-badge\n[codecov]: https://codecov.io/gh/hacf-fr/meteofrance-api\n[pypi-shield]: https://img.shields.io/pypi/v/meteofrance-api?style=for-the-badge\n[pypi]: https://pypi.org/project/meteofrance-api/\n",
    'author': 'oncleben31',
    'author_email': 'oncleben31@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hacf-fr/meteofrance-api',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""Météo-France weather forecast python API."""
from typing import Any
from typing import Optional

from requests import Response
from requests import Session

from .const import METEOFRANCE_API_TOKEN
from .const import METEOFRANCE_API_URL
from .const import METEOFRANCE_WS_API_URL
from .const import METEONET_API_URL


class MeteoFranceSession(Session):
    """Session for Météo-France."""

    host: str = METEOFRANCE_API_URL

    def __init__(self, access_token: Optional[str] = None):
        """Initialize the auth."""
        self.access_token = access_token or METEOFRANCE_API_TOKEN
        Session.__init__(self)

    def request(  # type: ignore
        self, method: str, path: str, *args: Any, **kwargs: Any
    ) -> Response:
        """Make a request."""
        params_inputs = kwargs.pop("params", None)

        params = {"token": self.access_token}
        if params_inputs:
            params.update(params_inputs)

        kwargs["params"] = params
        response = super().request(method, f"{self.host}/{path}", *args, **kwargs)
        response.raise_for_status()

        return response


class MeteoFranceWSSession(MeteoFranceSession):
    """Session for Météo-France WS."""

    host: str = METEOFRANCE_WS_API_URL

    def __init__(self, access_token: Optional[str] = None):
        """Initialize the Météo-France WS."""
        super().__init__(access_token)


class MeteoNetSession(MeteoFranceSession):
    """Session for MétéoNet."""

    host: str = METEONET_API_URL

    def __init__(self, access_token: Optional[str] = None):
        """Initialize the MétéoNet."""
        super().__init__(access_token)
